package cz.upol.jj2.lab09.cars;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

/**
 * Controller pro práci  s osobami (CRUD operace)
 */
@RestController
public class PersonController {
    //spring bean s "databází" vytvořený v konfiguraci (viz AppConfiguration.java)
    //odkaz pomocí qualifier, ne jména metody
    //anotace @Autowired pro dependency injection
    // - naváže bean s odpovídajícím jménem do tohoto atributu
    @Autowired
    @Qualifier("prefilled")
    private SimpleDatabase db;

    // GET pro získání osoby z DB
    // anotace GetMapping určuje endpoint i metodu - GET
    // @RequestParam navíc říká, jaký GET parametr odpovídá očekávanému parametru
    // návratový typ ResponseEntity<Person> navíc říká, že vrátíme přímo nachystanou HTTP response,
    // v jejímž těle bude Person
    @GetMapping("/person")
    public ResponseEntity<Person> person(@RequestParam(value = "id") int id) {
        System.out.printf("Hledám osobu s id %d.", id);
        Optional<Person> p = db.personById(id);
        if (p.isEmpty()) {
            System.out.println("KO");
            return ResponseEntity.status(404).body(null); //nenalazeno
        }
        System.out.println("OK");
        return ResponseEntity.status(200).body(p.get());  //ok
    }

    //POST pro vytvoření nebo upravení osoby v DB.
    // Dala by se použít jen @PostMapping("/person"), ale obecná
    // @RequestMapping nabízí větší kontrolu.
    // Navíc anotace @RequestBody u parametru person říká, že se do person
    // má deserializovat obsah těla requestu.
    // (Bude tam json s názvy a typy odpovídajícími atributům Person.)
    @RequestMapping(
            value = "/person",
            method = RequestMethod.POST,
            headers = "Accept=application/json",
            produces = "application/json; charset=UTF-8")
    public ResponseEntity<Person> create(@RequestBody Person person){
        int id = person.getId();
        var oldPersonData = db.personById(id);
        System.out.println("Vytvářím:");
        System.out.println(person);
        if (!oldPersonData.isEmpty()) {
            System.out.println("Kolize ID.");
            return ResponseEntity.status(418).body(null);
        }
        if(!db.addPerson(person)) {
            System.out.println("ale nedopadlo to...");
            db.print();
            return ResponseEntity.status(500).body(null); //interní chyba serveru
        }
        System.out.println("OK");
        db.print();
        return ResponseEntity.status(201).body(person); //vytvořeno
    }

    //PUT pro upravení dané osoby
    @RequestMapping(
            value = "/person",
            method = RequestMethod.PUT,
            headers = "Accept=application/json",
            produces = "application/json; charset=UTF-8")
    public ResponseEntity update(@RequestBody Person person){
        int id = person.getId();
        var oldPersonData = db.personById(id);
        if (oldPersonData.isEmpty()) {
            System.out.printf("Osoba k úpravě nenalazena (id=%d)", id);
            System.out.println();
            return ResponseEntity.status(404).body(null);
        }
        var oPer = oldPersonData.get();
        System.out.println("Upravuji:");
        System.out.println(oPer);
        System.out.println(person);

        oPer.setBorn(person.getBorn());
        oPer.setName(person.getName());
        oPer.setSurname(person.getSurname());
        System.out.println("OK");
        db.print();
        return ResponseEntity.status(200).body(oPer); //update ok
    }

    // DELETE pro smazání osoby z db
    // @DeleteMapping je obdoba @GetMapping/@PostMapping
    // Všimněte si, že tady @RequestBody anotovaný parametr není objekt, ale primitivní typ.
    // V requset body tedy bude hodnota odpovídajícího typu (zkuste poslat něco jiného...).
    // Navíc se metoda nezabývá vrácením HTTP status kódů, tedy ji stačí jen
    // antotovat jako @ResponseBody a Spring už se sám postará o transformaci
    // výsledku do těla response (včetně volby vhodného formátu dle různých kriterií).
     @DeleteMapping("/person")
     @ResponseBody
     public Boolean delete(@RequestBody int id) {
        System.out.printf("Mažu osobu s id %d.", id);

        var res = db.dropPerson(id);
        System.out.println(res?"OK":"KO");
        db.print();
        return res;
      }

    // GET/POST pro získání všech osob
    // ukážeme si, jak @ResponseBody reaguje na různé požadavky na formát odpovědi
    // a zároveň, jak do projektu přidat další závislost (podporu pro xml response)
    @RequestMapping(value = "/persons",
    method={RequestMethod.POST,RequestMethod.GET})
    @ResponseBody
    public Person[] persons() {
        return db.getPersons().toArray(new Person[0]);
    }
}
